import UppyCore from "@uppy/core";
import Dashboard from "@uppy/dashboard";

export default class FileUploader {
  /**
   * FileUploader constructor.
   *
   * @param {HTMLElement} element
   *   The file uploader element.
   * @param {Object} settings
   *   The configuration options.
   */
  constructor(element, settings) {
    this.element = element;
    this.input = element.querySelector(`[name="${settings.name}[fids]"]`);

    // Prepare the configuration options.
    settings.options.instance.restrictions = {};
    if (settings.options.validators.limit > 0) {
      settings.options.instance.restrictions.maxNumberOfFiles =
        settings.options.validators.limit;
    }
    if (settings.options.validators.extensions) {
      settings.options.instance.restrictions.allowedFileTypes =
        settings.options.validators.extensions;
    }
    if (settings.options.validators.filesize) {
      settings.options.instance.restrictions.maxFileSize =
        settings.options.validators.filesize;
    }
    settings.options.plugin.target = element;
    settings.options.plugin.inline = true;
    settings.options.plugin.doneButtonHandler = null;
    settings.options.plugin.showRemoveButtonAfterComplete = true;
    settings.options.plugin.proudlyDisplayPoweredByUppy = false;

    // Apply the locale translations when available.
    if (settings.options.instance.locale) {
      const locale = Uppy.locales[settings.options.instance.locale];
      if (typeof locale !== "undefined") {
        settings.options.instance.locale = locale;
      } else {
        delete settings.options.instance.locale;
      }
    }

    this.settings = settings;
    this.initialize();
    this.instance.on("upload-success", this.onUploadSuccess.bind(this));
    this.instance.on("file-removed", this.onFileRemoved.bind(this));

    // Add existing files.
    settings.values.forEach((existingFile) => {
      const mockFile = new File([""], existingFile.name);
      Object.defineProperty(mockFile, "size", {
        value: parseInt(existingFile.size, 10),
      });

      // Use mock file to immediately display preview. Automatically
      // update previews of existing image files.
      const fileId = this.instance.addFile({
        name: existingFile.name,
        type: existingFile.type,
        data: mockFile,
      });
      this.instance.setFileState(fileId, {
        progress: { uploadComplete: true, uploadStarted: true },
      });
      this.instance.setFileMeta(fileId, { fid: existingFile.fid });

      if (existingFile.type.match(/^image\//)) {
        fetch(existingFile.url)
          .then((response) => response.blob())
          .then((blob) => {
            // Update image thumbnail preview.
            const file = this.instance.getFile(fileId);
            if (file) {
              file.data = blob;
              this.instance.emit("thumbnail:request", file);
            }
          });
      }
    });
  }

  /**
   * Start the plugins.
   */
  initialize() {
    this.instance = new UppyCore(this.settings.options.instance).use(
      Dashboard,
      this.settings.options.plugin
    );
  }

  /**
   * Get the current files.
   *
   * @return {string[]}
   *   Returns an array of files.
   */
  getFiles() {
    return this.input.value.trim().split(" ");
  }

  /**
   * Set the current files.
   *
   * @param {Array} values
   *   An array of files to set.
   */
  setFiles(values) {
    this.input.value = values.join(" ");
  }

  /**
   * Handle upload success event.
   *
   * @param {Object} file
   *   The file.
   * @param {Object} response
   *   The upload response.
   */
  onUploadSuccess(file, response) {
    const files = this.getFiles();
    files.push(response.body.value);
    this.setFiles(files);
  }

  /**
   * Handle remove file event.
   *
   * @param {Object} file
   *   The file.
   */
  onFileRemoved(file) {
    if (file.meta.fid) {
      this.setFiles(
        this.getFiles().filter((fid) => {
          return fid !== file.meta.fid;
        })
      );
    }
  }
}
